@@ -27,7 +27,6 @@
 #include <stdbool.h>
 
 #include "evdev.h"
-#include "filter.h"
 #include "timer.h"
 
 #define TOUCHPAD_HISTORY_LENGTH 4
@@ -104,9 +103,6 @@ enum tp_tap_state {
 	TAP_STATE_DRAGGING,
 	TAP_STATE_DRAGGING_WAIT,
 	TAP_STATE_DRAGGING_2,
-	TAP_STATE_MULTITAP,
-	TAP_STATE_MULTITAP_DOWN,
-	TAP_STATE_MULTITAP_PALM,
 	TAP_STATE_DEAD, /**< finger count exceeded */
 };
 
