@@ -663,6 +663,10 @@ void
 litest_button_scroll(struct litest_device *d,
 		     unsigned int button,
 		     double dx, double dy);
+void
+litest_button_scroll_locked(struct litest_device *d,
+			    unsigned int button,
+			    double dx, double dy);
 
 void
 litest_keyboard_key(struct litest_device *d,
@@ -747,6 +751,10 @@ struct libinput_event_tablet_pad *
 litest_is_pad_strip_event(struct libinput_event *event,
 			  unsigned int number,
 			  enum libinput_tablet_pad_strip_axis_source source);
+struct libinput_event_tablet_pad *
+litest_is_pad_key_event(struct libinput_event *event,
+			unsigned int key,
+			enum libinput_key_state state);
 
 struct libinput_event_switch *
 litest_is_switch_event(struct libinput_event *event,
@@ -771,6 +779,10 @@ void
 litest_assert_only_typed_events(struct libinput *li,
 				enum libinput_event_type type);
 
+void
+litest_assert_no_typed_events(struct libinput *li,
+			      enum libinput_event_type type);
+
 void
 litest_assert_tablet_button_event(struct libinput *li,
 				  unsigned int button,
@@ -788,6 +800,10 @@ void
 litest_assert_pad_button_event(struct libinput *li,
 				    unsigned int button,
 				    enum libinput_button_state state);
+void
+litest_assert_pad_key_event(struct libinput *li,
+			    unsigned int key,
+			    enum libinput_key_state state);
 struct libevdev_uinput *
 litest_create_uinput_device(const char *name,
 			    struct input_id *id,
